{*------------------------------------------------------------------------------
  Create the structure of source code component from the result of the Parser
  All tools used by the Builder are in this unit.
  http://dephicodetodoc.sourceforge.net/
  Copyright TridenT 2003 - Under GNU GPL licence

  @Author    TridenT
  @Version   2003/12/01   TridenT   v0.0     Initial revision
-------------------------------------------------------------------------------}

unit Structure;

interface

uses Windows, SysUtils, Classes, Contnrs, ComCtrls, TAGComment, uOptions;

type
  /// Error result for the application
  TDOCError      = integer;
  /// List of all Binding method Keyword
  TMethodBind    = (mtStatic, mtDynamic, mtVirtual, mtAbstract);
  /// Set of all Binding method Keyword
  TMethodBindSet = set of TMethodBind;
  /// List of all method type Keyword
  TMethodType    = (mtProcedure, mtFunction, mtConstructor, mtDestructor);
  /// List of all method directive keyword (Delphi2005 and dotNET supported)
  TMethodDirectives = (mdExternal, mdPascal, mdSafecall, mdAbstract,
   mdAutomated, mdFar, mdStdcall, mdAssembler, mdInline, mdForward,
   mdVirtual, mdCdecl, mdMessage, mdName, mdRegister, mdDispId,
   mdNear, mdDynamic, mdExport, mdOverride, mdResident, mdLocal, mdOverload,
   mdReintroduce, mdDeprecated, mdLibrary, mdPlatform, mdStatic, mdFinal);
  /// Set of all method directive keyword
  TMethodDirectivesSet = set of TMethodDirectives;
  /// List of all class directive keyword (Delphi2005 and dotNET supported)
  TClassDirectives = (cdAbstract, cdSealed, cdHelper);
  /// Set of all class directive keyword
  TClassDirectivesSet = set of TClassDirectives;

  TDOCTemplate = class;
  /// Class factory based on TDOCTemplate
  TDOCTemplateClass = class of TDOCTemplate;
  TDOCTemplateList = class;
  TDOCMethod = class;

  TClassHierarchyList = TStringList;     /// List of Inherited Class and interface
  TDOCClassList    = TDOCTemplateList;   /// List of TDOCClass
  TDOCLinkList     = TDOCTemplateList;   /// List of TDOCLinks
  TDOCTypeList     = TDOCTemplateList;   /// List of TDOCTypes
  TDOCConstantList = TDOCTemplateList;   /// List of TDOCConst
  TDOCFieldList    = TDOCTemplateList;   /// List of TDOCField
  TDOCPropertyList = TDOCTemplateList;   /// List of TDOCPropertie
  TDOCEventList    = TDOCTemplateList;   /// List of TDOCEvent
  TDOCParameterList = TDOCTemplateList;  /// List of TDOCparameters
  TDOCMethodList   = TDOCTemplateList;   /// List of TDOCMethod
  TDOCVarList      = TDOCTemplateList;   /// List of TDOCVar
  TDOCFunctionList = TDOCTemplateList;   /// List of TDOCFunction
  TDOCUseUnitList  = TDOCTemplateList;   /// List of TDOCUseUnit
  TDOCUnitList     = TDOCTemplateList;

  ///  A link to a unit, class or another Maybe replace by an ObjectList with all DocTemplate !
  TDOCLink = class
  private
    Name: string;
    Link: TDOCTemplate;
  public
    constructor Create(const ln: string);
  end;

  /// Base class for Classes, members, ... in the final structure
  TDOCTemplate = class(TObject)
  private
    FRelatedParent: TDocTemplate; /// Logical Parent of this item
    FName: string;  /// Name of the item, often time an identifier
    FTAGHolderList: TTAGHolderList; /// Holder for all item's TAG
  public
    RelatedType: string;  /// TDOCLink;
    Links: TDOCLinkList;  /// of TDOCLinks
    BodySource: string; /// Source code of the expression
    property Name: string Read FName Write FName;
    property TAGHolderList: TTAGHolderList Read FTAGHolderList Write FTAGHolderList;
    property RelatedParent: TDocTemplate read FRelatedParent;
    constructor Create(const st: string; const rp: TDocTemplate); virtual;
    destructor Destroy; override;
    function AddLink(const ln: string): TDOCLink;
    function GetParentName: string;
    function GetFileName : string;virtual;
    function BuildPathName: string; virtual;
    function BuildRelativePathName: string; virtual;
    function CategoryType: TOutputCategoryList; virtual;
  end;

  /// Base class for TDOCTemplate containers
  TDOCTemplateList = class(TObjectList)
  protected
    function GetItem(const Index: integer): TDOCTemplate;
    procedure SetItem(const Index: integer; const item: TDOCTemplate);
  public
    property Items[const Index: integer]: TDOCTemplate Read GetItem Write SetItem;
      default;
    function FindItemByName(const st: string): TDOCTemplate;
    function FindOverloaded(const oname: string; oSourcecode:string): TDOCtemplate;
    function FindNextItemByName(const st: string; ItemStart:TDOCTemplate): TDOCTemplate;
    function NextItem(const Item: TDOCTemplate): TDOCTemplate;
    function PrevItem(const Item: TDOCTemplate): TDOCTemplate;
    function New(const aname: string; const DocParent: TDOCTemplate;
        const DocClass:TDOCTemplateClass): TDOCTemplate;virtual;
    end;

  /// Specialized TDOCTemplate class for Unit structure
  TDOCUnit = class(TDOCTemplate)
  private
    FFileName: TFilename;     /// Related Filename of the unit, generaly 'UnitName.pas' or 'ProgName.dpr'
    Classes: TDOCClassList; /// List of TDOCClasses
    Types: TDOCTypeList;  /// List of TDOCTypes
    Constants: TDOCConstantList;  /// oList f TDOCConst
    Vars: TDOCVarList;    /// List of TDOCInterface
    Functions: TDOCFunctionList; /// List of TDOCFunction
    FFlatClasses : TDOCClassList; /// Flat classes representation for this unit
  public
    property Filename: TFilename read FFilename write FFilename;
    property VarList : TDOCVarList read Vars write Vars;
    property TypeList : TDOCTypeList read Types write Types;
    property ConstantList : TDOCConstantList read Constants write Constants;
    property FunctionList : TDOCFunctionList read Functions write Functions;
    property ClassList : TDOCClassList read Classes write Classes;
    property FlatClassList : TDOCClassList read FFlatClasses write FFlatClasses;
    constructor Create(const st: string; const rp: TDocTemplate); override;
    destructor Destroy; override;
    procedure SortContent;
  end;

  /// Specialized TDOCTemplate class for Program structure
  TDOCProgram = class(TDOCUnit)
  private
    //FileName: string; /// Related Filename of the program, generally 'ProgramName.pas'
    UseUnits: TDOCUseUnitList; /// List of TDOCUseUnits
  public
    property UseUnitList: TDOCUseUnitList read UseUnits write UseUnits;
    constructor Create(const st: string; const rp: TDocTemplate); override;
    destructor Destroy; override;
  end;

  /// Specialized TDOCTemplate class for Library structure
  TDOCLibrary = class(TDOCProgram)
  private
    FExportsList: TStringList;  /// List of exported functions
  public
    property ExportsList: TStringList Read FExportsList write FExportsList;
    constructor Create(const st: string; const rp: TDocTemplate); override;
    destructor Destroy; override;
  end;

  /// Class factory based on TDOCUnit
  TDOCUnitClass = class of TDOCUnit;

  /// Specialized TDOCTemplate class for Member structure
  TDOCMember = class(TDOCTemplate)
  private
  public
    MemberVisibility: TMemberVisibility; /// Visibility of the Member
    StrictVisibility: Boolean; /// Strict visibility (apply only for private and public)
    IsClassMember: Boolean; /// Is it an object member or a class member
  end;

  /// Specialized TDOCMember class for Method structure with parameter support
  TDOCMethod = class(TDOCMember)
  private
    Parameters: TDOCParameterList; /// List of parameters in the method
    FOverloadID: WORD; /// Unique ID for overload methods
    function GetOverloadID: WORD;
    function ComputeOverloadID: WORD;
  public
    MethodType: TMethodType; /// Internal type of method (constructor, procedure, etc...)
    MethodDirectiveSet: TMethodDirectivesSet; /// Set of directive used in the method
    MethodBind: TMethodBind; /// Binding method
    property ParameterList: TDOCParameterList read Parameters write Parameters;
    property OverloadID : WORD read GetOverloadID;
    constructor Create(const st: string; const rp: TDocTemplate); override;
    destructor Destroy; override;
    function CategoryType: TOutputCategoryList; override;
    function BuildPathName: string; override;
    function GetFileName : string;override;
    function BuildRelativePathName: string; override;
  end;

  /// Specialized TDOCMember class for Field structure
  TDOCField = class(TDOCMember)
  public
    function CategoryType: TOutputCategoryList; override;
  end;

  /// Specialized TDOCMember class for Property structure
  TDOCProperty = class(TDOCMember)
  private
    FReadAssessor: TDOCTemplate;  /// The Read accessor
    FWriteAssessor: TDOCTemplate; /// The write accessor
  public
    HasReadSpecifier: boolean;  /// Property has Read keyword in declaration
    HasWriteSpecifier: boolean; /// Property has Write keyword in declaration
    ReadSpecifier: string;      /// Text for the read specifier
    WriteSpecifier: string;     /// Text for the write specifier
    property ReadAssessor : TDOCTemplate read FReadAssessor write FReadAssessor;
    property WriteAssessor : TDOCTemplate read FWriteAssessor write FWriteAssessor;
    constructor Create(const st: string; const rp: TDocTemplate); override;
    function CategoryType: TOutputCategoryList; override;
  end;

  /// Specialized TDOCMember class for Event structure
  TDOCEvent = class(TDOCMember)
  public
    function CategoryType: TOutputCategoryList; override;
  end;

  /// Specialized TDOCTemplate class for Parameter structure
  TDOCParameter = class(TDOCTemplate);

  /// Specialized TDOCTemplate class for Class structure
  TDOCVar = class(TDOCMember)
  public
    function CategoryType: TOutputCategoryList; override;
  end;

  /// Specialized TDOCTemplate class for Type structure
  TDOCType = class(TDocTemplate)
  public
    function CategoryType: TOutputCategoryList; override;
  end;

  /// Specialized TDOCTemplate class for Constant structure
  TDOCConstant = class(TDOCMember)
  public
    function CategoryType: TOutputCategoryList; override;
  end;

  /// Specialized TDOCTemplate class for Function structure
  TDOCFunction = class(TDOCMethod) //class(TDocTemplate);
  public
    function CategoryType: TOutputCategoryList; override;
  end;

  /// Specialized TDOCTemplate class for UseUnit structure
  TDOCUseUnit = class(TDocTemplate)
  public
    ExistFileName: Boolean; /// A filename is associated with the use unit's name
    FileName: TFileName;    /// Use unit's filename (eventually with path)
  end;

  /// Specialized TDOCTemplate class for Class structure
  TDOCClass = class(TDOCMember)
  private
    Fields: TDOCFieldList;   /// List of TDOCField
    Propertys: TDOCPropertyList; /// List of TDOCPropertie
    Methods: TDOCMethodList; /// List of TDOCMethod
    Events: TDOCEventList;   /// List of TDOCEvent
    Vars: TDOCVarList;  /// List of TDOCVar
    Constants: TDOCConstantList; /// List of TDOCConstant
    Classes: TDOCClassList; /// List of TDOCClass
    function GetMembersCount: integer;
    function GetNestedCount: integer;
  public
    ClassHierarchyList: TClassHierarchyList; /// Mainly parent class and Interfaces
    ClassDirectivesSet : TClassDirectivesSet; /// Set of directive used in the class
    property FieldList: TDOCFieldList read Fields write Fields;
    property PropertyList: TDOCPropertyList read Propertys write Propertys;
    property MethodList: TDOCMethodList read Methods write Methods;
    property EventList: TDOCEventList read Events write Events;
    property VarList: TDOCVarList read Vars write Vars;
    property ConstantList: TDOCConstantList read Constants write Constants;
    property ClassList: TDOCClassList read Classes write Classes;
    property NestedCount : integer read GetNestedCount;
    property MembersCount : integer read GetMembersCount;
    constructor Create(const st: string; const rp: TDOCTemplate); override;
    destructor Destroy; override;
    function FindOverloadedMethod(const mname: string; const msourcecode:string;
     const cname: string): TDOCtemplate;
    procedure SortContent;
    function CategoryType: TOutputCategoryList; override;
  end;

  /// Specialized TDOCClass for Interface structure
  TDOCInterface = class(TDOCClass)
  public
    function CategoryType: TOutputCategoryList; override;
  end;

  TDOCClassNode = TTreeNode;

  TDOCClassesTree = class(TTreeNodes)
  public
    constructor Create(AOwner: TCustomTreeView);overload;
    procedure AddClass(const DocClass: TDOCClass);
    function FindClass(const ClassName:string): TDOCClassNode;
  end;

  /// Specialized TDOCTemplate class for the whole structure itself
  TDOCStructure = class(TDOCTemplate)
  private
    Units: TDOCUnitList;      /// List of all units in the structure
    FDOCOptions: TDOCOptions; /// All Options. Beware! Shared Component.
    FProgram: TDOCProgram;    /// Program (DPR file) associate with structure (also a unit)
  public
    property UnitList: TDOCUnitList read Units write Units;
    property DocOptions: TDOCOptions Read FDOCOptions Write FDOCOptions;
    constructor Create(const st: string; const rp: TDocTemplate); override;
    destructor Destroy; override;
    procedure SetProgram(const DocProgram:TDOCProgram);
    function GetProgram: TDOCTemplate;
    procedure ConvertToTreeNode(const tnList: TTreeNodes);
    procedure SortContent;
    procedure Clear;
  end;

var
  /// List of Named visibility for Member used in members list
  MemberVisibilityName: TStringList;

{gnugettext: scan-all text-domain='DCTD_ignore' }
const
  /// Name of the method type
  MethodeTypeName: array[low(TMethodType)..High(TMethodType)] of string =
    ('Procedure', 'Function', 'Constructor', 'Destructor');  // Constant comment
  /// Symbol of all member visiblity
  MemberVisibilitySymbol: array[low(TMemberVisibility)..High(TMemberVisibility)]
    of string = ('-', '#', '+', '*', 'A');
  StrictVisibilitySymbol : string = 'S';
{gnugettext: reset }

implementation

uses JclStrings,JclMath, JclFileUtils;

{*------------------------------------------------------------------------------
  Comparison function for TDocTemplateList, used for sorting.
  @param Item1 Pointer (must be TDOCtemplate object) to first item to compare
  @param Item2 Pointer (must be TDOCtemplate object) to second item to compare
-------------------------------------------------------------------------------}
function DocTemplateListSortCompare(Item1, Item2: Pointer): integer;
begin
  Result := CompareText(TDOCtemplate(Item1).Name, TDOCTemplate(Item2).Name)
end;

/// Add a link to the Link List (NOT USED FOR THE MOMENT)
function TDOCTemplate.AddLink(const ln: string): TDOCLink;
begin
  Result := TDOCLink.Create(ln);
  Links.Add(Result);
end;

{*------------------------------------------------------------------------------
  Build a base filename with all parent's name separated with underscore
  It is mainly used to produce the output filename of this item's documentation.
  @return parent's name concatenation separated with underscore
-------------------------------------------------------------------------------}
function TDOCTemplate.BuildPathName: string;
var
  tmpDT: TDocTemplate;
begin
  Result := FName;
  tmpDT  := self;
  while tmpDT.FRelatedParent <> nil do
  begin
    tmpDT := tmpDT.FRelatedParent;
    insert(tmpDT.Name + PathSeparator {'_'}, Result, 1);
  end;
end;

{*------------------------------------------------------------------------------
  Build a base filename with last parent's name separated with underscore
  It is mainly used to produce link between items in documentation.
  @return Last parent's name concatenation separated with path separator
-------------------------------------------------------------------------------}
function TDOCTemplate.BuildRelativePathName: string;
var
  tmpDT: TDocTemplate;
begin
  Result := FName;
  tmpDT  := self;
  if tmpDT.FRelatedParent <> nil then
  begin
    tmpDT := tmpDT.FRelatedParent;
    insert(tmpDT.Name + '/' {'_'}, Result, 1);
  end;
end;

{*------------------------------------------------------------------------------
  Call Create to instantiate a TDOCTemplate object at runtime.
  Create allocates memory, and then initializes its properties.
  Generally, The owner of this item call create and add the reference in a list
  @param  st  Name of the new item (mainly its identifier)
  @param  rp  Related logical parent and owner
-------------------------------------------------------------------------------}
constructor TDOCTemplate.Create(const st: string; const rp: TDocTemplate);
begin
  inherited Create;
  FTAGHolderList := TTAGHolderList.Create(True);
  FName      := st;
  BodySource := '';
  Links      := TDOCLinkList.Create(False); // links are independants
  FRelatedParent := rp;
end;

/// Check if template item is required in the output documentation.
function TDOCTemplate.CategoryType: TOutputCategoryList;
begin
  // Just here for override !
  Exception.Create('Should never be called!');
  Result:=ocClasses;
end;

///  Destroys of an instance of a TDOCTemplate object
destructor TDOCTemplate.Destroy;
begin
  FreeAndNil(FTAGHolderList);
  FreeAndNil(Links);
  FRelatedParent := nil;
  inherited;
end;

{*------------------------------------------------------------------------------
  Return the name of the parent (owner)
  If no parent is assigned, then return the specified value '#NO-PARENT#'
  @return Parent and owner's item
-------------------------------------------------------------------------------}
function TDOCTemplate.GetParentName: string;
begin
  if FRelatedParent <> nil then
   Result := FRelatedParent.Name
  else
    Result := '#NO-PARENT#';
end;

{*------------------------------------------------------------------------------
  Call Create to instantiate a TDOCUnit object at runtime.
  Create allocates memory, and then initializes its properties.
  Generally, The owner of this item call create and add the reference in a list

  @param  st  Name of the new Unit (mainly its identifier)
  @param  rp  Related logical parent and owner, generally the Structure it self
-------------------------------------------------------------------------------}
constructor TDOCUnit.Create(const st: string; const rp: TDocTemplate);
begin
  inherited;
  Classes   := TDOCClassList.Create(True);
  Constants := TDOCConstantList.Create(True);
  Types     := TDOCTypeList.Create(True);
  Vars      := TDOCVarList.Create(True);
  Functions := TDOCFunctionList.Create(True);
  FFlatClasses := TDOCClassList.Create(False);
end;

///  Destroys of an instance of a TDOCUnit object
destructor TDOCUnit.Destroy;
begin
  FreeAndNil(FFlatClasses);
  FreeAndNil(Classes);
  FreeAndNil(Constants);
  FreeAndNil(Types);
  FreeAndNil(Vars);
  FreeAndNil(Functions);
  inherited;
end;

/// Sort all DocTemplate element of lists in alphabetical order
procedure TDOCUnit.SortContent;
var
  Index: integer;
begin
  Classes.Sort(DocTemplateListSortCompare);
  for Index := 0 to Classes.Count - 1 do
  begin
    TDOCClass(Classes.Items[Index]).SortContent
  end;

  Constants.Sort(DocTemplateListSortCompare);
  Functions.Sort(DocTemplateListSortCompare);
  Types.Sort(DocTemplateListSortCompare);
  Vars.Sort(DocTemplateListSortCompare);
end;

{*------------------------------------------------------------------------------
  Call Create to instantiate a TDOCCLass object at runtime.
  Create allocates memory, and then initializes its properties.
  Generally, The owner of this item call create and add the reference in a list

  @param  st  Name of the new Class (mainly its identifier)
  @param  rp  Related logical parent and owner, the unit it written into
-------------------------------------------------------------------------------}
constructor TDOCClass.Create(const st: string; const rp: TDocTemplate);
begin
  inherited;
  ClassHierarchyList := TClassHierarchyList.Create;
  ClassDirectivesSet := [];
  Events    := TDOCEventList.Create(True);
  Fields    := TDOCFieldList.Create(True);
  Methods   := TDOCMethodList.Create(True);
  Propertys := TDOCPropertyList.Create(True);
  Vars := TDOCVarList.Create(True);
  Constants := TDOCConstantList.Create(True);
  Classes := TDOCClassList.Create(True);
end;

///  Destroys of an instance of a TDOCCLass object
destructor TDOCClass.Destroy;
begin
  FreeAndNil(Classes);
  FreeAndNil(Constants);
  FreeAndNil(Vars);
  FreeAndNil(Events);
  FreeAndNil(Fields);
  FreeAndNil(Methods);
  FreeAndNil(Propertys);
  FreeAndNil(ClassHierarchyList);
  inherited;
end;

/// Sort all DocTemplate element of lists in alphabetical order
procedure TDOCClass.SortContent;
begin
  Classes.Sort(DocTemplateListSortCompare);
  Constants.Sort(DocTemplateListSortCompare);
  Vars.Sort(DocTemplateListSortCompare);
  Fields.Sort(DocTemplateListSortCompare);
  Methods.Sort(DocTemplateListSortCompare);
  Events.Sort(DocTemplateListSortCompare);
  Propertys.Sort(DocTemplateListSortCompare);
end;

{*------------------------------------------------------------------------------
  Find a TDOCTemplate Object corresponding to the method's name and sourcecode.
  This prevent from returning only the first pethod found with the same name
   if the function is overloaded (same name, but different parameter).
  @param mname Method's name to find
  @param msourcecode Method's Source code
  @param cname Class name for the method
  @return Pointer to the found method 
-------------------------------------------------------------------------------}
function TDOCClass.FindOverloadedMethod(const mname: string; const msourcecode:string;
 const cname: string): TDOCtemplate;
const
  ClassDotDelimiter: string = ' . ';
var
  tmpWantedMethod: string;
  tmpFound: boolean;
begin
  assert(assigned(Methods));
  tmpFound:=false;
  // Delete the Classname and the dot
  tmpWantedMethod:=Uppercase(StringReplace(msourcecode,cName+ClassDotDelimiter
   ,'',[rfIgnoreCase]));
  Result:=nil;
  repeat
    Result := MethodList.FindNextItemByName(mName,Result);
    if assigned(Result) then
     // Test if Wanted method is a substring of the found one
     if Pos(tmpWantedMethod,UpperCase(Result.BodySource))<>0 then tmpFound:=true;
  until (Result=nil) or (tmpFound=true);
end;

{*------------------------------------------------------------------------------
  Link the Program (Delphi DPR) to the structure.
  @param  DocProgram TDOCProgram to link to the current structure
-------------------------------------------------------------------------------}
procedure TDOCStructure.SetProgram(const DocProgram:TDOCProgram);
begin
  if assigned(FProgram) then raise Exception.Create('Program already exists')
  else
  begin
    FProgram := DocProgram;
  end;
end;

{*------------------------------------------------------------------------------
  Return the linked Program (Delphi DPR)
  @result  TDOCProgram linked to the current structure
-------------------------------------------------------------------------------}
function TDOCStructure.GetProgram: TDOCTemplate;
begin
  result:=FProgram;
end;

/// Clear all units from the structure
procedure TDOCStructure.Clear;
begin
  Units.Clear;
  // FProgram was freed before as unit !
  FProgram := nil;
end;

{*------------------------------------------------------------------------------
  Converts the structure into a treenode
  The structure has units, which hold classes, which hold fields and methods
   and events and propertys ... So, it can be represented has a treenode.
  The TreeNode wil be cleared before filling.
  @param      tnList An initiazied TreeNode
-------------------------------------------------------------------------------}
procedure TDOCStructure.ConvertToTreeNode(const tnList: TTreeNodes);
var
  tnRoot, tnUnit, tnClass: TTreeNode;
  IndexUnit, IndexClass: integer;
  tmpClass, tmpUnit: TDOCTemplate;
begin
  Assert(tnList <> nil);
  // Clear the List
  tnList.Clear;
  // Add Root project
  tnRoot := tnList.Add(nil, FName);
  // Add Units
  for IndexUnit := 0 to Units.Count - 1 do
  begin
    tmpUnit     := UnitList[IndexUnit];
    tnUnit      := tnList.AddChild(tnRoot, tmpUnit.FName);
    tnUnit.Data := tmpUnit;
    for IndexClass := 0 to (tmpUnit as TDOCUnit).ClassList.Count - 1 do
    begin
      tmpClass     := (tmpUnit as TDOCUnit).ClassList.Items[IndexClass];
      tnClass      := tnList.AddChild(tnUnit, tmpClass.FName);
      tnClass.Data := tmpClass;
    end;
  end;
end;


{*------------------------------------------------------------------------------
  Call Create to instantiate a TDOCStructure object at runtime.
  Create allocates memory, and then initializes its properties.
  Generally, there is only one reference per application.
  @param  st  Name of the new Strcutre (mainly its identifier)
  @param  rp  Related logical parent and owner, generally a null value !
-------------------------------------------------------------------------------}
constructor TDOCStructure.Create(const st: string; const rp: TDocTemplate);
begin
  inherited;
  FName := 'Project1';
  Units := TDOCUnitList.Create(True);
  FProgram:=nil;
end;

///  Destroys of an instance of a TDOCStructure object and free its content
destructor TDOCStructure.Destroy;
begin
  FreeAndNil(Units);
  inherited;
end;

/// Sort all DocTemplate element of lists in alphabetical order
procedure TDOCStructure.SortContent;
var
  Index: integer;
begin
  Units.Sort(DocTemplateListSortCompare);
  for Index := 0 to Units.Count - 1 do
  begin
    TDOCUnit(Units.Items[Index]).SortContent
  end;
end;

{ TDOCTemplateList }

{*------------------------------------------------------------------------------
  Find a TDOCTemplate reference in the list by its name
  @param  st Name of the item to find
  @return Item reference if successfull, nil otherwise
-------------------------------------------------------------------------------}
function TDOCTemplateList.FindItemByName(const st: string): TDOCTemplate;
var
  Tmp:   TDOCTemplate;
  Index: integer;
begin
  Result := nil;
  Index  := 1;
  while ((Index <= Count) and (Result = nil)) do
  begin
    Tmp := Items[Index - 1];
    if Assigned(Tmp) then
    begin
      //if Tmp.FName = st then
      if SameText(Tmp.FName,st) then
      begin
        Result := Tmp
      end
    end;
    Inc(Index);
  end;
end;

{*------------------------------------------------------------------------------
  Find the next overloaded Template matching the SourceCode
  @param  oname Name of the item to find
  @param oSourcecode Source code to match with
  @return The Template if found, otherwise NIL
-------------------------------------------------------------------------------}
function TDOCTemplateList.FindOverloaded(const oname: string;
 oSourcecode:string): TDOCtemplate;
var
  tmpWanted: string;
  tmpFound: boolean;
begin
  tmpFound:=false;
  // transform to uppercase for comparison
  tmpWanted:=Uppercase(oSourcecode);
  Result:=nil;
  repeat
    Result := FindNextItemByName(oName,Result);
    if assigned(Result) then
     // Test if Wanted method is a substring of the found one
     if Pos(tmpWanted,UpperCase(Result.BodySource))<>0 then tmpFound:=true;
  until (Result=nil) or (tmpFound=true);
end;

{*------------------------------------------------------------------------------
  Find a TDOCTemplate reference in the list by its name
  @param  st Name of the item to find
  @return Item reference if successfull, nil otherwise
-------------------------------------------------------------------------------}
function TDOCTemplateList.FindNextItemByName(const st: string; ItemStart:TDOCTemplate): TDOCTemplate;
var
  Tmp:   TDOCTemplate;
  Index: integer;
begin
  Result := nil;
  // Exit if no members
  if Count<1 then exit;
  // Start from StartItem parameter of First item
  if ItemStart=nil then tmp:=self.First as TDOCTemplate
  else tmp:=self.NextItem(ItemStart);
  if tmp=nil then exit;

  Index  := self.IndexOf(tmp)+1;
  while ((Index <= Count) and (Result = nil)) do
  begin
    Tmp := Items[Index - 1];
    if Assigned(Tmp) then
      if SameText(Tmp.FName,st) then Result := Tmp;
    Inc(Index);
  end;
end;

{*------------------------------------------------------------------------------
  Get the reference of the DocTemplate specified by its index in the Items List
  @param  index Index in the DocTemplate List
  @return DocTemplate reference if successfull, nil otherwise
-------------------------------------------------------------------------------}
function TDOCTemplateList.GetItem(const Index: integer): TDOCTemplate;
begin
  Result := inherited GetItem(Index) as TDOCTemplate;
end;

{*------------------------------------------------------------------------------
  Get the next DocTemplate item in the DocTemplate List
  @param  Item  Base item to find the next one
  @return DocTemplate Next item reference if successfull, nil otherwise
-------------------------------------------------------------------------------}
function TDOCTemplateList.NextItem(const Item: TDOCTemplate): TDOCTemplate;
var
  Index: integer;
begin
  Index := IndexOf(Item);
  if Index < (Count - 1) then
  begin
    Result := Items[Index + 1]
  end
  else begin
    Result := nil
  end;
end;

{*------------------------------------------------------------------------------
  Get the previous DocTemplate item in the DocTemplate List
  @param  Item  Base item to find the previous one
  @return DocTemplate Previous item reference if successfull, nil otherwise
-------------------------------------------------------------------------------}
function TDOCTemplateList.PrevItem(const Item: TDOCTemplate): TDOCTemplate;
var
  Index: integer;
begin
  Index := IndexOf(Item);
  if Index > 0 then
  begin
    Result := Items[Index - 1]
  end
  else begin
    Result := nil
  end;
end;

{*------------------------------------------------------------------------------
  Set the reference of the DocTemplate specified by its index in the Items List
  @param  index Index in the DocTemplate List
  @param  item  reference of the TDOCTemplate to set
-------------------------------------------------------------------------------}
procedure TDOCTemplateList.SetItem(const Index: integer; const item: TDOCTemplate);
begin
  inherited SetItem(index, item as TObject);
end;

{*------------------------------------------------------------------------------
  Call Create to instantiate a TDOCLink object at runtime.
  Create allocates memory, and then initializes its properties.
  Generally, The owner of this item call create and add the reference in a list
  @param  ln  Name of the new Link (mainly its identifier)
-------------------------------------------------------------------------------}
constructor TDOCLink.Create(const ln: string);
begin
  Name := ln;
  Link := nil;
end;

{*------------------------------------------------------------------------------
  Call Create to instantiate a TDOCMethod object at runtime.
  Create allocates memory, and then initializes its properties.
  Generally, The owner of this item call create and add the reference in a list

  @param  st  Name of the new Method (mainly its identifier)
  @param  rp  Related logical parent and owner, generally the class it depend
-------------------------------------------------------------------------------}
constructor TDOCMethod.Create(const st: string; const rp: TDocTemplate);
begin
  inherited;
  Parameters := TDOCParameterList.Create(True);
end;

{*------------------------------------------------------------------------------
  Check if template item is required in the output documentation.
  @return Category list for a method
-------------------------------------------------------------------------------}
function TDOCMethod.CategoryType: TOutputCategoryList;
begin
  Result:=ocMethods;
end;

///  Destroys of an instance of a TDOCMethod object
destructor TDOCMethod.Destroy;
begin
  FreeAndNil(Parameters);
  inherited;
end;

/// Check if template item is required in the output documentation.
function TDOCInterface.CategoryType: TOutputCategoryList;
begin
  Result:=ocClasses;
end;

/// Check if template item is required in the output documentation.
function TDOCField.CategoryType: TOutputCategoryList;
begin
  Result:=ocFields;
end;

/// Check if template item is required in the output documentation.
function TDOCProperty.CategoryType: TOutputCategoryList;
begin
  Result:=ocProperties;
end;

/// Check if template item is required in the output documentation.
function TDOCEvent.CategoryType: TOutputCategoryList;
begin
  Result:=ocEvents;
end;

/// Check if template item is required in the output documentation.
function TDOCVar.CategoryType: TOutputCategoryList;
begin
  Result:=ocVariables;
end;

/// Check if template item is required in the output documentation.
function TDOCType.CategoryType: TOutputCategoryList;
begin
  Result:=ocTypes;
end;

/// Check if template item is required in the output documentation.
function TDOCConstant.CategoryType: TOutputCategoryList;
begin
  Result:=ocConstants;
end;

/// Check if template item is required in the output documentation.
function TDOCFunction.CategoryType: TOutputCategoryList;
begin
  Result:=ocFunctions;
end;

/// Check if template item is required in the output documentation.
function TDOCClass.CategoryType: TOutputCategoryList;
begin
  Result:=ocClasses;
end;

/// Build Pathname with CRC inside !
function TDOCMethod.BuildPathName: string;
var
  tmpPathName: string;
begin
  tmpPathName:=inherited BuildPathName;
  Result:=tmpPathName+'_'+IntToHex(GetOverloadID,4);
end;

/// Build Relative Pathname with CRC inside !
function TDOCMethod.BuildRelativePathName: string;
var
  tmpPathName: string;
begin
  tmpPathName:=inherited BuildRelativePathName;
  Result:=tmpPathName+'_'+IntToHex(GetOverloadID,4);
end;

{ TDOCProgram }

{*------------------------------------------------------------------------------
  Call Create to instantiate a TDOCProgram object at runtime.
  Create allocates memory, and then initializes its properties.
  Generally, The owner of this item call create and add the reference in a list
  @param  st  Name of the new Program (mainly its identifier)
  @param  rp  Related logical parent and owner, generally the global Structure
-------------------------------------------------------------------------------}
constructor TDOCProgram.Create(const st: string; const rp: TDocTemplate);
begin
  inherited;
  UseUnits:=TDOCTemplateList.Create(true);
end;

/// Destroy the TDOCProgram instance
destructor TDOCProgram.Destroy;
begin
  FreeAndNil(UseUnits);
  inherited;
end;

{*------------------------------------------------------------------------------
  Call Create to instantiate a TDOCProperty object at runtime.
  Create allocates memory, and then initializes its properties.
  Generally, The owner of this item call create and add the reference in a list
  @param  st  Name of the new Property (mainly its identifier)
  @param  rp  Related logical parent and owner, generally the Class it depends
-------------------------------------------------------------------------------}
constructor TDOCProperty.Create(const st: string; const rp: TDocTemplate);
begin
  inherited;
  HasReadSpecifier:=false;
  HasWriteSpecifier:=false;
end;

{ TDOCLibrary }

{*------------------------------------------------------------------------------
  Call Create to instantiate a TDOCLibrary object at runtime.
  Create allocates memory, and then initializes its properties.
  Generally, The owner of this item call create and add the reference in a list
  @param  st  Name of the new Library (mainly its identifier)
  @param  rp  Related logical parent and owner, generally the global Structure
-------------------------------------------------------------------------------}
constructor TDOCLibrary.Create(const st: string; const rp: TDocTemplate);
begin
  inherited;
  FExportsList:=TStringList.Create;
end;

/// Destroy a TDOCLibrary instance
destructor TDOCLibrary.Destroy;
begin
  FreeAndNil(FExportsList);
  inherited;
end;

{ TDOCClassesTree }

constructor TDOCClassesTree.Create(AOwner: TCustomTreeView);
begin
  inherited;
  //AddClass('TObject');
  //AddClass('IInterface');
end;

procedure TDOCClassesTree.AddClass(const DocClass: TDOCClass);
var
  ParentClassNode: TTreeNode;
begin
  // Find node with parent's name
  ParentClassNode:=FindClass(DocClass.ClassHierarchyList.Strings[0]);
  // Create Class Node
  AddChildObject(ParentClassNode,DocClass.Name,DocClass)
end;

function TDOCClassesTree.FindClass(const ClassName: string): TDOCClassNode;
var
  IndexClassNode: Integer;
  ClassNodeFound: boolean;
begin
  IndexClassNode:=0;
  ClassNodeFound:=false;
  // Search through tree
  while ((not ClassNodeFound) and (Count<IndexClassNode)) do
  begin
    if Item[IndexClassNode].Text=ClassName then ClassNodeFound:=true
    else inc(IndexClassNode);
  end;
  if ClassNodeFound then Result:=Item[IndexClassNode]
   else Result:=nil;
end;

function TDOCTemplateList.New(const aname: string; const DocParent: TDOCTemplate;
    const DocClass:TDOCTemplateClass): TDOCTemplate;
begin
  Result:= DocClass.Create(aname,DocParent);
  Add(Result);
end;

/// Return Field, Method, Event and Property count
function TDOCClass.GetMembersCount: integer;
begin
  Result := Fields.Count + Propertys.Count + Methods.Count + Events.Count;
end;

/// Return Class, variable, constant count
function TDOCClass.GetNestedCount: integer;
begin
  Result := Constants.Count + Vars.Count + Classes.Count;
end;

function TDOCTemplate.GetFileName: string;
begin
  Result:=FName;
end;

function TDOCMethod.GetFileName: string;
var
  tmpName : string;
begin
  tmpName := inherited GetFileName;
  Result:=tmpName+'_'+IntToHex(GetOverloadId,4);
end;

function TDOCMethod.GetOverloadID: WORD;
begin
  if FOverloadID <> 0 then Result := FOverloadID
  else
  begin
    Result := ComputeOverloadID;
    FOverloadID := Result;
  end;
end;

function TDOCMethod.ComputeOverloadID: WORD;
var
  pba:PByteArray;
begin
  pba:=PByteArray(@self.BodySource[1]);
  Result:=JclMath.CRC16(pba[0],Length(BodySource),0);
end;

initialization
  MemberVisibilityName := TStringList.Create;
  MemberVisibilityName.Add('PRIVATE');
  MemberVisibilityName.Add('PROTECTED');
  MemberVisibilityName.Add('PUBLIC');
  MemberVisibilityName.Add('PUBLISHED');
  MemberVisibilityName.Add('AUTOMATED');

finalization
  FreeAndNil(MemberVisibilityName);
end.
