{*------------------------------------------------------------------------------
  User interface of DelphiCodeToDoc
  http://dephicodetodoc.sourceforge.net/
  Copyright TridenT 2003 - Under GNU GPL licence
  @Author    TridenT
  @Version   2003/12/01   TridenT   v0.0     Initial revision
-------------------------------------------------------------------------------}

unit fMainForm;

interface

uses
  Classes, Forms, ImgList, Controls, StdActns, JvDialogActns, ActnList,
  XPStyleActnCtrls, ActnMan, ActnMenus, ToolWin, ActnCtrls, ComCtrls,
  JvStatusBar, JvComponent, JvLookOut, SysUtils, StdCtrls, JvInspector,
  JvComCtrls, JvGnugettext, JvExComCtrls, JvExControls;

type
  TMainForm = class(TForm)
    JvExpress1: TJvExpress;
    ebInformation: TJvExpressButton;
    ebConfiguration: TJvExpressButton;
    ebProjectFiles: TJvExpressButton;
    ebReference: TJvExpressButton;
    ebMessages: TJvExpressButton;
    ImageListSmall: TImageList;
    ImageListLarge: TImageList;
    ActionManager1: TActionManager;
    ActionNewproject: TAction;
    ActionQuickWizardProject: TAction;
    ActionOpenProject: TJvOpenFileAction;
    ActionSaveAsProject: TJvSaveFileAction;
    FileExit1: TFileExit;
    ActionAbout: TAction;
    ActionAddFiles: TAction;
    ActionCheck: TAction;
    ActionBuild: TAction;
    ActionInformation: TAction;
    ActionConfiguration: TAction;
    ActionProject: TAction;
    ActionReference: TAction;
    ActionMessages: TAction;
    StatusBar1: TStatusBar;
    JvPageControl1: TJvPageControl;
    tsInformation: TTabSheet;
    tsConfiguration: TTabSheet;
    tsProjectFiles: TTabSheet;
    tsReference: TTabSheet;
    tsMessages: TTabSheet;
    mDebug: TMemo;
    JvInspector1: TJvInspector;
    JvInspectorDotNETPainter1: TJvInspectorDotNETPainter;
    mUser: TMemo;
    tvReference: TJvTreeView;
    LabelMustCheckBefore: TLabel;
    ActionCloseProject: TAction;
    ActionLanguageEnglish: TAction;
    ActionToolBar1: TActionToolBar;
    ActionMainMenuBar1: TActionMainMenuBar;
    Memo1: TMemo;
    ActionCheckAndBuild: TAction;
    ActionViewOutput: TAction;
    ActionPurgeOutputFolder: TAction;
    ActionSaveProject: TAction;
    ActionClearMessages: TAction;
    ActionFullWizardProject: TAction;
    procedure ActionAboutExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ActionInformationExecute(Sender: TObject);
    procedure ActionConfigurationExecute(Sender: TObject);
    procedure ActionProjectExecute(Sender: TObject);
    procedure ActionReferenceExecute(Sender: TObject);
    procedure ActionMessagesExecute(Sender: TObject);
    procedure ActionFullWizardProjectExecute(Sender: TObject);
    procedure ActionCheckExecute(Sender: TObject);
    procedure ActionBuildExecute(Sender: TObject);
    procedure ActionCloseProjectExecute(Sender: TObject);
    procedure ActionManager1Update(Action: TBasicAction; var Handled: boolean);
    procedure ActionSaveAsProjectAccept(Sender: TObject);
    procedure ActionOpenProjectAccept(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure AskChangeLanguage(Sender: TObject);
    procedure ActionCheckAndBuildExecute(Sender: TObject);
    procedure ActionViewOutputExecute(Sender: TObject);
    procedure ActionPurgeOutputFolderExecute(Sender: TObject);
    procedure ActionSaveProjectExecute(Sender: TObject);
    procedure ActionSaveAsProjectJvSaveDialogShow(Sender: TObject);
    procedure ActionClearMessagesExecute(Sender: TObject);
    procedure ActionQuickWizardProjectExecute(Sender: TObject);
  private
    { Private declarations }
    EnableActions: Boolean;
    procedure AddTranslationsToMenu;
    procedure AddConfigurationPropertys;
    procedure ChangeChkState(const Item: TJvCustomInspectorItem);
    procedure iieBoolsAsChecks(Sender: TObject);
    procedure UpdateReferences;
  public
    { Public declarations }
    procedure AddUserMessage(const st: string);
    procedure AddDebugMessage(const st: string);
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

uses fAbout, uDocProject, JvInspExtraEditors, TypInfo, fWizardProject
  , JvMemo, JclShell, uOptions, Dialogs, StrUtils, Graphics, uSharedProcess
  , JclFileUtils, Windows, frmNiceWizard;

{gnugettext: scan-all text-domain='DCTD_ignore' }
const
  HTM_FILE_EXT :string = '.htm';
{gnugettext: reset }

{*------------------------------------------------------------------------------
  This is the reply to the selection of Menu>About
  @Param Sender default Sender Object
-------------------------------------------------------------------------------}
procedure TMainForm.ActionAboutExecute(Sender: TObject);
var
  AboutBox: TAboutBox;
begin
  AboutBox := TAboutBox.Create(self);
  AboutBox.ShowModal;
  AboutBox.Release;
end;

// Add translations found with SearchTranslations to menu
procedure TMainForm.AddTranslationsToMenu;
var
  IndexLanguage: integer;
  tmpAction:     TAction;
  LanguageFound: boolean;
begin
  LanguageFound := False;
  // Initialization to prevent compilation warning
  tmpAction:=nil;
  for IndexLanguage := 0 to AvailableLanguages.Count - 1 do
  begin
    // Create Action
    tmpAction := TAction.Create(ActionManager1);
    tmpAction.ActionList := ActionManager1;
    tmpAction.Assign(ActionLanguageEnglish);
    //tmpAction.Name:='ActionLanguage'+AvailableLanguages.Strings[IndexLanguage];
    tmpAction.AutoCheck := False;
    tmpAction.Caption := AvailableLanguages.Strings[IndexLanguage] +
      ' (' + AvailableLanguagesCode.Strings[IndexLanguage] + ')';
    tmpAction.Tag := IndexLanguage;
    tmpAction.GroupIndex := 1;
    // Add Action to ActionMenu
    ActionManager1.AddAction(tmpAction
      , ActionManager1.FindItemByAction(ActionLanguageEnglish));
    // Autodetect language used by comparing CurrentLanguage and Available language
    if AnsiStartsText(AvailableLanguagesCode.Strings[IndexLanguage]
      , GetCurrentLanguage) then
    begin
      tmpAction.Checked := True;
      LanguageFound     := True;
    end;
  end;
  // if autodetect failed, Set English (last language in list) as standard
  Assert(Assigned(tmpAction));
  if not LanguageFound then
    tmpAction.Checked := True;

  // Hide TAction Template
  ActionLanguageEnglish.Visible := False;
end;


procedure TMainForm.FormCreate(Sender: TObject);
begin
  // Ignore some classes for translation
  IgnoreTranslations;
  // Search for existing Translations
  SearchTranslations;
  AddTranslationsToMenu;
  // Translate the form
  TranslateComponent(self);
  //TranslateComponent(JvInspector1);
  self.Caption := Application.Title;
  ActionInformationExecute(self);
  // Set divider to 50% of inspector's width
  JvInspector1.Divider := JvInspector1.Width div 2;
  // Enable actions
  EnableActions:=True;
end;

procedure TMainForm.ActionInformationExecute(Sender: TObject);
begin
  ebInformation.Down := True;
  tsInformation.Show;
end;

procedure TMainForm.ActionConfigurationExecute(Sender: TObject);
begin
  ebConfiguration.Down := True;
  tsConfiguration.Show;
end;

procedure TMainForm.ActionProjectExecute(Sender: TObject);
begin
  ebProjectFiles.Down := True;
  tsProjectFiles.Show;
end;

procedure TMainForm.ActionReferenceExecute(Sender: TObject);
begin
  ebReference.Down := True;
  tsReference.Show;
end;

procedure TMainForm.ActionMessagesExecute(Sender: TObject);
begin
  ebMessages.Down := True;
  tsMessages.Show;
end;

procedure TMainForm.ChangeChkState(const Item: TJvCustomInspectorItem);
var
  I: integer;
begin
  if Item is TJvInspectorBooleanItem then
  begin
    TJvInspectorBooleanItem(Item).ShowAsCheckbox := DocProject.DocOptions.BoolsAsChecks
  end;
  for I := 0 to Item.Count - 1 do
  begin
    ChangeChkState(Item[I])
  end;
end;

function AddProperty(const Cat: TJvInspectorCustomCategoryItem;
  const Prop: TPropertyDescription): TJvCustomInspectorItem;
begin
  Result := TJvInspectorPropData.New(Cat, DocProject.DocOptions,
    GetPropInfo(DocProject.DocOptions, Prop.iName));
  Result.ReadOnly := Prop.iReadOnly;
  // Must force translation of Prop.iCaption, maybe because of resourcestrings
  //  include in a const table !
  if Prop.iReadOnly then
  begin
    Result.DisplayName := gettext(Prop.iCaption) + gettext(' (ReadOnly)')
  end
  else begin
    Result.DisplayName := gettext(Prop.iCaption)
  end;

  if Prop.iName = pdNameBoolsAsChecks then
  begin
    Result.OnValueChanged :=
      MainForm.iieBoolsAsChecks
  end;
  MainForm.ChangeChkState(Result);
end;

procedure TMainForm.AddConfigurationPropertys;
var
  InspCat:   TJvInspectorCustomCategoryItem;
  InspCatInput : TJvInspectorCustomCategoryItem;
  IndexProp: integer;
begin
  JvInspector1.BeginUpdate;
  JvInspector1.Root.SortKind := iskNone;
  JvInspector1.SelectedIndex := 0;
  // APPLICATION
  InspCat := TJvInspectorCustomCategoryItem.Create(JvInspector1.Root, nil);
  InspCat.DisplayName := ccApplicationOptions;
  InspCat.Expanded := True;
  for IndexProp := low(C_APPLICATION_PROPERTYS_DESCRIPTION)
    to high(C_APPLICATION_PROPERTYS_DESCRIPTION) do
  begin
    AddProperty(InspCat, C_APPLICATION_PROPERTYS_DESCRIPTION[IndexProp])
  end;
  // INPUT
  InspCatInput := TJvInspectorCustomCategoryItem.Create(JvInspector1.Root, nil);
  InspCatInput.DisplayName := ccInputOptions;
  InspCatInput.Expanded := True;
  for IndexProp := low(C_INPUT_PROPERTYS_DESCRIPTION)
    to high(C_INPUT_PROPERTYS_DESCRIPTION) do
  begin
    AddProperty(InspCatInput, C_INPUT_PROPERTYS_DESCRIPTION[IndexProp])
  end;
  // PROPERTIES COMMENT
  InspCat := TJvInspectorCustomCategoryItem.Create(InspCatInput, nil);
  InspCat.DisplayName := ccPropertiesComment;
  InspCat.Expanded := True;
  for IndexProp := low(C_PROPERTIES_COMMENT)
    to high(C_PROPERTIES_COMMENT) do
  begin
    AddProperty(InspCat, C_PROPERTIES_COMMENT[IndexProp])
  end;

  // JAVADOC PREFIX
  InspCat := TJvInspectorCustomCategoryItem.Create(InspCatInput, nil);
  InspCat.DisplayName := ccJavaDocPrefixOptions;
  InspCat.Expanded := True;
  for IndexProp := low(C_JAVADOC_PREFIX_PROPERTYS_DESCRIPTION)
    to high(C_JAVADOC_PREFIX_PROPERTYS_DESCRIPTION) do
  begin
    AddProperty(InspCat, C_JAVADOC_PREFIX_PROPERTYS_DESCRIPTION[IndexProp])
  end;

  // Files
  InspCat := TJvInspectorCustomCategoryItem.Create(InspCatInput, nil);
  InspCat.DisplayName := ccFilesList;
  InspCat.Expanded := True;
  for IndexProp := low(C_FILES_LIST_PROPERTYS_DESCRIPTION)
    to high(C_FILES_LIST_PROPERTYS_DESCRIPTION) do
  begin
    AddProperty(InspCat, C_FILES_LIST_PROPERTYS_DESCRIPTION[IndexProp])
  end;

  // OUTPUT
  InspCat := TJvInspectorCustomCategoryItem.Create(JvInspector1.Root, nil);
  InspCat.DisplayName := ccOutputOptions;
  InspCat.Expanded := True;
  for IndexProp := low(C_OUTPUT_PROPERTYS_DESCRIPTION)
    to high(C_OUTPUT_PROPERTYS_DESCRIPTION) do
  begin
    AddProperty(InspCat, C_OUTPUT_PROPERTYS_DESCRIPTION[IndexProp])
  end;
  // JAVADOC SUPPORT
  InspCat := TJvInspectorCustomCategoryItem.Create(InspCat, nil);
  InspCat.DisplayName := ccJavaDocSupportOptions;
  InspCat.Expanded := True;
  for IndexProp := low(C_JAVADOC_SUPPORT_PROPERTYS_DESCRIPTION)
    to high(C_JAVADOC_SUPPORT_PROPERTYS_DESCRIPTION) do
  begin
    AddProperty(InspCat, C_JAVADOC_SUPPORT_PROPERTYS_DESCRIPTION[IndexProp])
  end;
  // End of build
  JvInspector1.EndUpdate;
  //ReTranslateComponent(JvInspector1);
end;

procedure TMainForm.iieBoolsAsChecks(Sender: TObject);
begin
  JvInspector1.BeginUpdate;
  try
    ChangeChkState(JvInspector1.Root);
  finally
    JvInspector1.EndUpdate;
  end;
end;

procedure TMainForm.ActionFullWizardProjectExecute(Sender: TObject);
begin
  FullWizardForm := TFullWizardForm.Create(self);
  if not assigned(DocProject) then
  begin
    DocProject := TDOCProject.Create;
    DocProject.DocOptions.SetDefaultValue;
    DocProject.SetDestinationMessages(mUser.Lines, mDebug.Lines);
    DocProject.State := psNew;
  end;
  FullWizardForm.DocOptionsToWizard(Docproject.DocOptions);
  if FullWizardForm.ShowModal = mrOk then
  begin
    FullWizardForm.WizardToDocOption(Docproject.DocOptions);
    AddConfigurationPropertys;
    ActionConfigurationExecute(self);
  end
  else
  begin
    // Cancel the project
    FreeAndNil(DocProject);
    AddDebugMessage(L_WizardCanceled);
  end;
  FullWizardForm.Release;
end;

procedure TMainForm.ActionCheckExecute(Sender: TObject);
begin
  EnableActions:=FALSE;
  // Go to the Messages panel
  ActionMessages.Execute;
  // Check if Project can be built
  if DocProject.Check then
  begin
    LabelMustCheckBefore.Visible := False;
    // UpdateReference
    UpdateReferences;
    tvReference.Visible := True;
    //ActionBuild.Enabled := True;
    AddUserMessage(L_CheckedSuccess);
    // Go to the Messages panel
    ActionMessages.Execute;
  end
  else  begin
    LabelMustCheckBefore.Visible := True;
    // UpdateReference
    UpdateReferences;
    tvReference.Visible := False;
    AddUserMessage(L_CheckingAborted);
    // Go to the Messages panel
    ActionMessages.Execute;
    // Show Error Panel
    MessageDlg(MSG_ASSERTION_ERROR, mtError, [mbOK], 0);
  end;
  EnableActions:=TRUE;
end;

procedure TMainForm.ActionBuildExecute(Sender: TObject);
begin
  EnableActions:=FALSE;
  // Build Project documentation
  Assert(DocProject.isChecked);
  // Go to the Messages panel
  ActionMessagesExecute(self);
  // Verify if all files are built !
  if DocProject.Build then
  begin
    AdduserMessage(L_BuildSuccess);
    AddDebugMessage(L_BuildDone);
    // Launch the browser ?
    if DocProject.DocOptions.LaunchBrowser then
     ActionViewOutputExecute(Sender);
  end
  else  begin
    AdduserMessage(L_BuildError);
    // Show Error panel
    MessageDlg(MSG_ASSERTION_ERROR, mtError, [mbOK], 0);
  end;
  EnableActions:=TRUE;
end;

procedure TMainForm.AddDebugMessage(const st: string);
begin
  mDebug.Lines.Add(st);
end;

procedure TMainForm.AddUserMessage(const st: string);
begin
  mUser.Lines.Add(st);
  StatusBar1.SimpleText := st;
end;

procedure RecurseSetCheck(const tn: TTreeNode; const Value: boolean);
var
  Index: integer;
  jvtn:  TJvTreeNode;
begin
  assert(tn is TJvTreeNode);
  jvtn := tn as TJvTreeNode;
  jvtn.Checked := Value;
  // Issue : Checkmarks doesn't work !!!
  //jvtn.Bold:=Value;
  for Index := 0 to jvtn.Count - 1 do
  begin
    RecurseSetCheck(jvtn.Item[Index], Value)
  end;
end;

// List project's units and classes in a TreeView
procedure TMainForm.UpdateReferences;
begin
  // TODO: save reference informations if exists
  // Clear top Reference if exist  (inside convert function)
  //if tvReference.TopItem <> nil then tvReference.TopItem.Delete;
  DocProject.GetStructure.ConvertToTreeNode(tvReference.Items);
  tvReference.FullExpand;
  // Add checkmark to all Nodes
  RecurseSetCheck(tvReference.Items.GetFirstNode, True);
  // TODO : retrieve reference information if exist
end;

procedure TMainForm.ActionCloseProjectExecute(Sender: TObject);
begin
  // Free Configuration list
  JvInspector1.Root.Clear;
  // Close project
  if Assigned(DocProject) then
  begin
    FreeAndNil(DocProject)
  end;
  tvReference.Items.Clear;
  AddUserMessage(L_ProjectClosed);
end;

procedure TMainForm.ActionManager1Update(Action: TBasicAction; var Handled: boolean);
begin
  if EnableActions then
  begin
    ActionMainMenuBar1.Enabled:=True;
    ActionToolBar1.Enabled:=True;
  end
  else
  begin
    ActionMainMenuBar1.Enabled:=False;
    ActionToolBar1.Enabled:=False;
  end;

   if assigned(DocProject) then
   begin
    case DocProject.State of
      psNew:
      begin
        ActionQuickWizardProject.Enabled := False;
        ActionFullWizardProject.Enabled := False;
        ActionCloseProject.Enabled := True;
        ActionCheck.Enabled := False;
        ActionBuild.Enabled := False;
        ActionCheckAndBuild.Enabled := True;
        ActionOpenProject.Enabled := True;
        ActionSaveProject.Enabled := True;
        ActionSaveAsProject.Enabled := True;
        ActionViewOutput.Enabled := True;
        ActionPurgeOutputFolder.Enabled := True;
      end;
      psSaved:
      begin
        ActionQuickWizardProject.Enabled := False;
        ActionFullWizardProject.Enabled := False;
        ActionCloseProject.Enabled := True;
        ActionCheck.Enabled := False;
        ActionBuild.Enabled := False;
        ActionCheckAndBuild.Enabled := True;
        ActionOpenProject.Enabled := True;
        ActionSaveProject.Enabled := True;
        ActionSaveAsProject.Enabled := True;
        ActionViewOutput.Enabled := True;
        ActionPurgeOutputFolder.Enabled := True;
      end;
      psModified:
      begin
        ActionQuickWizardProject.Enabled := False;
        ActionFullWizardProject.Enabled := False;
        ActionCloseProject.Enabled := True;
        ActionCheck.Enabled := False;
        ActionBuild.Enabled := False;
        ActionCheckAndBuild.Enabled := True;
        ActionOpenProject.Enabled := True;
        ActionSaveProject.Enabled := True;
        ActionSaveAsProject.Enabled := True;
        ActionViewOutput.Enabled := True;
        ActionPurgeOutputFolder.Enabled := True;
      end;
      else // Unknown state !
      begin
        ActionQuickWizardProject.Enabled := False;
        ActionFullWizardProject.Enabled := False;
        ActionCloseProject.Enabled := False;
        ActionCheck.Enabled := False;
        ActionBuild.Enabled := False;
        ActionCheckAndBuild.Enabled := False;
        ActionOpenProject.Enabled := False;
        ActionSaveProject.Enabled := False;
        ActionSaveAsProject.Enabled := False;
        ActionViewOutput.Enabled := False;
        ActionPurgeOutputFolder.Enabled := False;
      end;
    end
   end
   else // Project doesn't exist or Actions disables
   begin
    ActionQuickWizardProject.Enabled := True;
    ActionFullWizardProject.Enabled := True;
    ActionCloseProject.Enabled := False;
    ActionCheck.Enabled := False;
    ActionBuild.Enabled := False;
    ActionCheckAndBuild.Enabled := False;
    ActionOpenProject.Enabled := True;
    ActionSaveProject.Enabled := False;
    ActionSaveAsProject.Enabled := False;
    ActionViewOutput.Enabled := False;
    ActionPurgeOutputFolder.Enabled := False;
   end;
  handled := True;
end;

// TODO - Extract SaveToStream option !
procedure TMainForm.ActionSaveAsProjectAccept(Sender: TObject);
var
  FS:  TFileStream;
  Res: TRWOptionsError;
begin
  Assert(assigned(DocProject));
  FS := TFileStream.Create(ActionSaveAsProject.Dialog.FileName, fmCreate);
  try
    Res := DocProject.DocOptions.SaveToStream(FS);
  finally
    FS.Free;
  end;
  if Res = oeNoError then
  begin
    AddUserMessage(L_ProjectSavedSuccess);
    MessageDlg(L_ProjectSavedSuccess, mtInformation, [mbOK], 0);
    DocProject.FileName:=ActionSaveAsProject.Dialog.FileName;
    DocProject.State:=psSaved;
  end
  else
  begin
    AddUserMessage(L_ProjectSavingError);
    AddDebugMessage(L_ProjectSavingError + ':' + RWOptionsErrorStrings[Res]);
    MessageDlg(L_ProjectSavingError, mtError, [mbOK], 0);
  end;
end;

// TODO - Extract LoadToStream option !
procedure TMainForm.ActionOpenProjectAccept(Sender: TObject);
var
  FS:  TFileStream;
  Res: TRWOptionsError;
begin
  if assigned(DocProject) then
  begin
    ActionCloseProjectExecute(Sender)
  end;
  // Create New Project
  DocProject := TDOCProject.Create;
  DocProject.DocOptions.SetDefaultValue;

  FS := TFileStream.Create(ActionOpenProject.Dialog.FileName, fmOpenRead);
  try
    Res := DocProject.DocOptions.LoadFromStream(FS);
  finally
    FS.Free;
  end;
  // Ask for a redraw of Configuration Tree
  iieBoolsAsChecks(self);
  JvInspector1.Invalidate;
  if Res = oeNoError then
  begin
    AddConfigurationPropertys;
    DocProject.SetDestinationMessages(mUser.Lines, mDebug.Lines);
    DocProject.FileName:=ActionOpenProject.Dialog.FileName;
    DocProject.State:=psSaved;
    MessageDlg(L_ProjectLoadedSuccess, mtInformation, [mbOK], 0);
    AddUserMessage(L_ProjectLoadedSuccess);
    // Go to the Messages panel
    ActionConfigurationExecute(self);
  end
  else
  begin
    // Create new project aborted
    AddUserMessage(L_ProjectLoadingError);
    AddDebugMessage(L_ProjectLoadingError + ': ' + RWOptionsErrorStrings[Res]);
    MessageDlg(L_ProjectLoadingError + ': ' + RWOptionsErrorStrings[Res]
     , mtError, [mbOK], 0);
    FreeAndNil(DocProject);
  end;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  ActionCloseProjectExecute(Sender);
end;

procedure TMainForm.AskChangeLanguage(Sender: TObject);
var
  LCode: string;
begin
  if (Sender is TAction) then
  begin
    if (Sender as TAction).Tag = 999 then
    begin
      LCode := 'en'
    end
    else begin
      LCode := AvailableLanguagesCode.Strings[(Sender as TAction).Tag]
    end;
    UseLanguage(LCode);
    ReTranslateComponent(self);
    Self.Caption := Application.Title;
    //ReTranslateComponent(JvInspector1);
  end;
end;

procedure TMainForm.ActionCheckAndBuildExecute(Sender: TObject);
begin
  // Add delimiter for this action
  AddDebugMessage(StringOfChar('-',160));
  // Check the project
  ActionCheckExecute(Sender);
  // and build if checked ok
  if DocProject.isChecked then
   ActionBuildExecute(Sender);
end;

procedure TMainForm.ActionViewOutputExecute(Sender: TObject);
begin
  AddUserMessage(L_LaunchBrowser);
  ShellExecEx(DocProject.OutputMainFileName);
end;

procedure TMainForm.ActionPurgeOutputFolderExecute(Sender: TObject);
begin
  assert(assigned(DocProject));
  if MessageBox(0, PAnsiChar(Format(L_COnfirmDeleteHTMfiles,[Docproject.DocOptions.OutputFolder]))
   ,'DelphiCodeToDoc', MB_ICONWARNING or MB_OKCANCEL)=idOK then
    if DocProject.PurgeOutputFolder then Sysutils.Beep;
  ActionMessagesExecute(Sender);
end;


procedure TMainForm.ActionSaveProjectExecute(Sender: TObject);
begin
  // Verify if project has already been saved (have a defined name)
  case DocProject.State of
   psNew:
   begin
    // Call SaveAs Action
    ActionSaveAsProject.Execute;
   end;
   psSaved:
   begin
    // Nothing to do, project always saved !
    ActionSaveAsProject.Dialog.FileName:=DocProject.FileName;
    ActionSaveAsProjectAccept(self);    
   end;
   psModified:
   begin
    ActionSaveAsProject.Dialog.FileName:=DocProject.FileName;
    ActionSaveAsProjectAccept(self);
   end;
   else
    raise Exception.Create('Unknown project state !')
  end;
end;

procedure TMainForm.ActionSaveAsProjectJvSaveDialogShow(Sender: TObject);
begin
  // Set project's Name before showing Dialog in Project is new !
  // THIS DON'T WORK !!!
  if (DocProject.State=psNew) and (DocProject.Filename='') then
   ActionSaveAsProject.Dialog.FileName:=Docproject.DocOptions.Name+'.dcd';
end;

/// Clear Output messages (Debug and user)
procedure TMainForm.ActionClearMessagesExecute(Sender: TObject);
begin
  mDebug.Clear;
  mUser.Clear;
end;

procedure TMainForm.ActionQuickWizardProjectExecute(Sender: TObject);
begin
  WizardProjectForm := TWizardProjectForm.Create(self);
  if WizardProjectForm.ShowModal = mrOk then
  begin
    DocProject := TDOCProject.Create;
    DocProject.DocOptions.SetDefaultValue;
    // get informations from wizard
    if WizardProjectForm.IsProjectSelected then
      Docproject.DocOptions.FilesIncludeList.Add(WizardProjectForm.GetProjectFile)
    else
      Docproject.DocOptions.FilesIncludeList.Add(WizardProjectForm.GetRootDirectory);
    DocProject.DocOptions.OutputFolder := WizardProjectForm.GetOutputDirectory;
    DocProject.DocOptions.Name := WizardProjectForm.GetProjectName;
    AddConfigurationPropertys;
    DocProject.SetDestinationMessages(mUser.Lines, mDebug.Lines);
    AddUserMessage(L_WizardSuccess);
    DocProject.State := psNew;
    // Go to the Messages panel
    ActionConfigurationExecute(self);
  end
  else
  begin
    AddDebugMessage(L_WizardCanceled)
  end;
  WizardProjectForm.Release;
end;

initialization
  TJvInspectorColorItem.RegisterAsDefaultItem;
  TJvInspectorTImageIndexItem.RegisterAsDefaultItem;
end.
